'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VSRunner = void 0;
const browser_1 = require("../browser");
const fs = __importStar(require("fs-extra"));
const mocha_1 = __importDefault(require("mocha"));
const glob_1 = require("glob");
const path = __importStar(require("path"));
const yaml = __importStar(require("js-yaml"));
const sanitize_filename_1 = __importDefault(require("sanitize-filename"));
const selenium_webdriver_1 = require("selenium-webdriver");
const os = __importStar(require("os"));
/**
 * Mocha runner wrapper
 */
class VSRunner {
    mocha;
    chromeBin;
    customSettings;
    codeVersion;
    cleanup;
    tmpLink = path.join(os.tmpdir(), 'extest-code');
    releaseType;
    constructor(bin, codeVersion, customSettings = {}, cleanup = false, releaseType, config) {
        const conf = this.loadConfig(config);
        this.mocha = new mocha_1.default(conf);
        this.chromeBin = bin;
        this.customSettings = customSettings;
        this.codeVersion = codeVersion;
        this.cleanup = cleanup;
        this.releaseType = releaseType;
    }
    /**
     * Set up mocha suite, add vscode instance handling, run tests
     * @param testFilesPattern glob pattern of test files to run
     * @param logLevel The logging level for the Webdriver
     * @return The exit code of the mocha process
     */
    runTests(testFilesPattern, code, logLevel = selenium_webdriver_1.logging.Level.INFO, resources) {
        return new Promise(resolve => {
            let self = this;
            let browser = new browser_1.VSBrowser(this.codeVersion, this.releaseType, this.customSettings, logLevel);
            const testFiles = new Set();
            for (const pattern of testFilesPattern) {
                const universalPattern = pattern.replace(/'/g, '');
                (0, glob_1.globSync)(universalPattern).reverse()
                    .forEach((val) => testFiles.add(val));
            }
            testFiles.forEach((file) => this.mocha.addFile(file));
            this.mocha.suite.afterEach(async function () {
                if (this.currentTest && this.currentTest.state !== 'passed') {
                    try {
                        const filename = (0, sanitize_filename_1.default)(this.currentTest.fullTitle());
                        await browser.takeScreenshot(filename);
                    }
                    catch (err) {
                        console.log('Screenshot capture failed.', err);
                    }
                }
            });
            this.mocha.suite.beforeAll(async function () {
                this.timeout(180000);
                const start = Date.now();
                const binPath = process.platform === 'darwin' ? await self.createShortcut(code.getCodeFolder(), self.tmpLink) : self.chromeBin;
                await browser.start(binPath);
                await browser.openResources(...resources);
                await browser.waitForWorkbench();
                await new Promise((res) => { setTimeout(res, 3000); });
                console.log(`Browser ready in ${Date.now() - start} ms`);
                console.log('Launching tests...');
            });
            this.mocha.suite.afterAll(async function () {
                this.timeout(180000);
                await browser.quit();
                if (process.platform === 'darwin') {
                    if (await fs.pathExists(self.tmpLink)) {
                        try {
                            fs.unlinkSync(self.tmpLink);
                        }
                        catch (err) {
                            console.log(err);
                        }
                    }
                }
                code.uninstallExtension(self.cleanup);
            });
            this.mocha.run((failures) => {
                process.exitCode = failures ? 1 : 0;
                if (process.exitCode) {
                    console.log('\x1b[33m%s\x1b[0m', `INFO: Screenshots of failures can be found in: ${browser.getScreenshotsDir()}\n`);
                }
                resolve(process.exitCode);
            });
        });
    }
    async createShortcut(src, dest) {
        try {
            await fs.ensureSymlink(src, dest, 'dir');
        }
        catch (err) {
            return this.chromeBin;
        }
        const dir = path.parse(src);
        const segments = this.chromeBin.split(path.sep);
        const newSegments = dest.split(path.sep);
        let found = false;
        for (let i = 0; i < segments.length; i++) {
            if (!found) {
                found = segments[i] === dir.base;
            }
            else {
                newSegments.push(segments[i]);
            }
        }
        return path.join(dir.root, ...newSegments);
    }
    loadConfig(config) {
        const defaultFiles = ['.mocharc.js', '.mocharc.json', '.mocharc.yml', '.mocharc.yaml'];
        let conf = {};
        let file = config;
        if (!config) {
            file = path.resolve('.');
            for (let i = 0; i < defaultFiles.length; i++) {
                if (fs.existsSync(path.join(file, defaultFiles[i]))) {
                    file = path.join(file, defaultFiles[i]);
                    break;
                }
            }
        }
        if (file && fs.existsSync(file) && fs.statSync(file).isFile()) {
            console.log(`Loading mocha configuration from ${file}`);
            if (/\.(yml|yaml)$/.test(file)) {
                try {
                    conf = yaml.load(fs.readFileSync(file, 'utf-8'));
                }
                catch (err) {
                    console.log('Invalid mocha configuration file, will be ignored');
                }
            }
            else if (/\.(js|json)$/.test(file)) {
                try {
                    conf = require(path.resolve(file));
                }
                catch (err) {
                    console.log('Invalid mocha configuration file, will be ignored');
                }
            }
            else {
                console.log('Unsupported mocha configuration file extension, make sure to use .js, .json, .yml or .yaml file');
            }
        }
        return conf;
    }
}
exports.VSRunner = VSRunner;
//# sourceMappingURL=runner.js.map